@@ -11,6 +11,7 @@
 #include <QQmlNetworkAccessManagerFactory>
 #include <QQuickStyle>
 #include <QQuickWindow>
+#include <QtQml/QQmlExtensionPlugin>
 
 #ifdef Q_OS_ANDROID
 #include <QGuiApplication>
@@ -29,6 +30,11 @@
 #ifdef HAVE_WINDOWSYSTEM
 #include <KWindowSystem>
 #endif
+
+#if __has_include("KCrash")
+#include <KCrash>
+#endif
+
 #include <KLocalizedContext>
 #include <KLocalizedString>
 
@@ -44,6 +50,7 @@
 #include "neochatconfig.h"
 #include "roommanager.h"
 #include "windowcontroller.h"
+#include "sharehandler.h"
 
 #ifdef HAVE_RUNNER
 #include "runner.h"
@@ -144,7 +151,7 @@ int main(int argc, char *argv[])
     about.addAuthor(i18n("Tobias Fella"), i18n("Maintainer"), QStringLiteral("tobias.fella@kde.org"), QStringLiteral("https://tobiasfella.de"));
     about.addAuthor(i18n("James Graham"), i18n("Maintainer"), QStringLiteral("james.h.graham@protonmail.com"));
     about.addCredit(i18n("Black Hat"), i18n("Original author of Spectral"), QStringLiteral("bhat@encom.eu.org"));
-    about.addCredit(i18n("Alexey Rusakov"), i18n("Maintainer of Quotient"), QStringLiteral("Kitsune-Ral@users.sf.net"));
+    about.addCredit(i18n("Alexey Rusakov"), i18n("Maintainer of libQuotient"), QStringLiteral("Kitsune-Ral@users.sf.net"));
     about.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
     about.setOrganizationDomain("kde.org");
 
@@ -160,6 +167,10 @@ int main(int argc, char *argv[])
     KAboutData::setApplicationData(about);
     QGuiApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("org.kde.neochat")));
 
+#if __has_include("KCrash")
+    KCrash::initialize();
+#endif
+
     initLogging();
 
     Connection::setEncryptionDefault(true);
@@ -191,6 +202,9 @@ int main(int argc, char *argv[])
     parser.addOption(dbusActivatedOption);
 #endif
 
+    QCommandLineOption shareOption(QStringLiteral("share"), i18n("Share a URL to Matrix"), QStringLiteral("text"));
+    parser.addOption(shareOption);
+
     about.setupCommandLine(&parser);
     parser.process(app);
     about.processCommandLine(&parser);
@@ -218,36 +232,43 @@ int main(int argc, char *argv[])
     KDBusService service(KDBusService::Unique);
 #endif
 
-    qml_register_types_org_kde_neochat();
-    qmlRegisterSingletonInstance("org.kde.neochat.config", 1, 0, "Config", NeoChatConfig::self());
-    qmlRegisterSingletonInstance("org.kde.neochat.accounts", 1, 0, "AccountRegistry", &Controller::instance().accounts());
+    Q_IMPORT_QML_PLUGIN(org_kde_neochat_settingsPlugin)
+    Q_IMPORT_QML_PLUGIN(org_kde_neochat_timelinePlugin)
+    Q_IMPORT_QML_PLUGIN(org_kde_neochat_devtoolsPlugin)
+    Q_IMPORT_QML_PLUGIN(org_kde_neochat_loginPlugin)
 
-    qmlRegisterUncreatableType<KeyVerificationSession>("com.github.quotient_im.libquotient", 1, 0, "KeyVerificationSession", {});
+    qml_register_types_org_kde_neochat();
 
     QQmlApplicationEngine engine;
 
 #ifdef HAVE_KDBUSADDONS
     service.connect(&service,
-                    &KDBusService::activateRequested,
-                    &RoomManager::instance(),
-                    [&engine](const QStringList &arguments, const QString &workingDirectory) {
-                        Q_UNUSED(workingDirectory);
+        &KDBusService::activateRequested,
+        &RoomManager::instance(),
+        [&engine](const QStringList &arguments, const QString &workingDirectory) {
+            Q_UNUSED(workingDirectory);
 
-                        QWindow *window = windowFromEngine(&engine);
-                        KWindowSystem::updateStartupId(window);
+            QWindow *window = windowFromEngine(&engine);
+            KWindowSystem::updateStartupId(window);
 
-                        WindowController::instance().showAndRaiseWindow(QString());
+            WindowController::instance().showAndRaiseWindow(QString());
 
-                        // Open matrix uri
-                        if (arguments.isEmpty()) {
-                            return;
-                        }
-                        auto args = arguments;
-                        args.removeFirst();
-                        for (const auto &arg : args) {
-                            RoomManager::instance().resolveResource(arg);
-                        }
-                    });
+            // Open matrix uri
+            if (arguments.isEmpty()) {
+                return;
+            }
+
+            auto args = arguments;
+            args.removeFirst();
+            if (args.length() == 2 && args[0] == "--share"_ls) {
+                ShareHandler::instance().setText(args[1]);
+                return;
+            }
+
+            for (const auto &arg : args) {
+                RoomManager::instance().resolveResource(arg);
+            }
+        });
 #endif
 
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
@@ -260,15 +281,19 @@ int main(int argc, char *argv[])
         });
     }
 
+    if (parser.isSet("share"_ls)) {
+        ShareHandler::instance().setText(parser.value(shareOption));
+    }
+
     engine.addImageProvider(QLatin1String("mxc"), MatrixImageProvider::create(&engine, &engine));
     engine.addImageProvider(QLatin1String("blurhash"), new BlurhashImageProvider);
 
-    engine.load(QUrl(QStringLiteral("qrc:/org/kde/neochat/qml/main.qml")));
+    engine.loadFromModule("org.kde.neochat", "Main");
     if (engine.rootObjects().isEmpty()) {
         return -1;
     }
 
-    if (!parser.positionalArguments().isEmpty()) {
+    if (!parser.positionalArguments().isEmpty() && !parser.isSet("share"_ls)) {
         RoomManager::instance().setUrlArgument(parser.positionalArguments()[0]);
     }
 
