@@ -9,17 +9,7 @@
 #include "roommanager.h"
 #include "spacehierarchycache.h"
 
-#include <QDebug>
-#if QT_VERSION < QT_VERSION_CHECK(6, 6, 0)
-#ifndef Q_OS_ANDROID
-#include <QDBusConnection>
-#include <QDBusInterface>
-#include <QDBusMessage>
-#endif
-#endif
-
 #include <KLocalizedString>
-#include <QGuiApplication>
 
 using namespace Quotient;
 
@@ -28,37 +18,6 @@ Q_DECLARE_METATYPE(Quotient::JoinState)
 RoomListModel::RoomListModel(QObject *parent)
     : QAbstractListModel(parent)
 {
-    const auto collapsedSections = NeoChatConfig::collapsedSections();
-    for (auto collapsedSection : collapsedSections) {
-        m_categoryVisibility[collapsedSection] = false;
-    }
-
-    connect(this, &RoomListModel::highlightCountChanged, this, [this]() {
-#if QT_VERSION < QT_VERSION_CHECK(6, 6, 0)
-#ifndef Q_OS_ANDROID
-        // copied from Telegram desktop
-        const auto launcherUrl = "application://org.kde.neochat.desktop"_ls;
-        // Gnome requires that count is a 64bit integer
-        const qint64 counterSlice = std::min(m_highlightCount, 9999);
-        QVariantMap dbusUnityProperties;
-
-        if (counterSlice > 0) {
-            dbusUnityProperties["count"_ls] = counterSlice;
-            dbusUnityProperties["count-visible"_ls] = true;
-        } else {
-            dbusUnityProperties["count-visible"_ls] = false;
-        }
-
-        auto signal = QDBusMessage::createSignal("/com/canonical/unity/launcherentry/neochat"_ls, "com.canonical.Unity.LauncherEntry"_ls, "Update"_ls);
-
-        signal.setArguments({launcherUrl, dbusUnityProperties});
-
-        QDBusConnection::sessionBus().send(signal);
-#endif // Q_OS_ANDROID
-#else
-        qGuiApp->setBadgeNumber(m_highlightCount);
-#endif // QT_VERSION_CHECK(6, 6, 0)
-    });
     connect(&SpaceHierarchyCache::instance(), &SpaceHierarchyCache::spaceHierarchyChanged, this, [this]() {
         Q_EMIT dataChanged(index(0, 0), index(rowCount(), 0), {IsChildSpaceRole});
     });
@@ -66,12 +25,12 @@ RoomListModel::RoomListModel(QObject *parent)
 
 RoomListModel::~RoomListModel() = default;
 
-Quotient::Connection *RoomListModel::connection() const
+NeoChatConnection *RoomListModel::connection() const
 {
     return m_connection;
 }
 
-void RoomListModel::setConnection(Connection *connection)
+void RoomListModel::setConnection(NeoChatConnection *connection)
 {
     if (connection == m_connection) {
         return;
@@ -127,7 +86,6 @@ void RoomListModel::doResetModel()
         doAddRoom(room);
     }
     endResetModel();
-    refreshNotificationCount();
 }
 
 NeoChatRoom *RoomListModel::roomAt(int row) const
@@ -153,7 +111,7 @@ void RoomListModel::connectRoomSignals(NeoChatRoom *room)
         refresh(room, {DisplayNameRole});
     });
     connect(room, &Room::unreadStatsChanged, this, [this, room] {
-        refresh(room, {NotificationCountRole, HighlightCountRole});
+        refresh(room, {ContextNotificationCountRole, HasHighlightNotificationsRole});
     });
     connect(room, &Room::notificationCountChanged, this, [this, room] {
         refresh(room);
@@ -176,44 +134,6 @@ void RoomListModel::connectRoomSignals(NeoChatRoom *room)
     connect(room, &Room::pendingEventMerged, this, [this, room] {
         refresh(room, {SubtitleTextRole});
     });
-    connect(room, &Room::unreadStatsChanged, this, &RoomListModel::refreshNotificationCount);
-    connect(room, &Room::highlightCountChanged, this, &RoomListModel::refreshHighlightCount);
-}
-
-int RoomListModel::notificationCount() const
-{
-    return m_notificationCount;
-}
-
-int RoomListModel::highlightCount() const
-{
-    return m_highlightCount;
-}
-
-void RoomListModel::refreshNotificationCount()
-{
-    int count = 0;
-    for (auto room : std::as_const(m_rooms)) {
-        count += room->notificationCount();
-    }
-    if (m_notificationCount == count) {
-        return;
-    }
-    m_notificationCount = count;
-    Q_EMIT notificationCountChanged();
-}
-
-void RoomListModel::refreshHighlightCount()
-{
-    int count = 0;
-    for (auto room : std::as_const(m_rooms)) {
-        count += room->highlightCount();
-    }
-    if (m_highlightCount == count) {
-        return;
-    }
-    m_highlightCount = count;
-    Q_EMIT highlightCountChanged();
 }
 
 void RoomListModel::updateRoom(Room *room, Room *prev)
@@ -288,6 +208,9 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
     if (role == DisplayNameRole) {
         return room->displayName();
     }
+    if (role == EscapedDisplayNameRole) {
+        return room->displayName().toHtmlEscaped();
+    }
     if (role == AvatarRole) {
         return room->avatarMediaId();
     }
@@ -298,36 +221,13 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
         return room->topic();
     }
     if (role == CategoryRole) {
-        if (room->joinState() == JoinState::Invite) {
-            return NeoChatRoomType::Invited;
-        }
-        if (room->isFavourite()) {
-            return NeoChatRoomType::Favorite;
-        }
-        if (room->isLowPriority()) {
-            return NeoChatRoomType::Deprioritized;
-        }
-        if (room->isDirectChat()) {
-            return NeoChatRoomType::Direct;
-        }
-        const RoomCreateEvent *creationEvent = room->creation();
-        if (!creationEvent) {
-            return NeoChatRoomType::Normal;
-        }
-        QJsonObject contentJson = creationEvent->contentJson();
-        QJsonObject::const_iterator typeIter = contentJson.find("type"_ls);
-        if (typeIter != contentJson.end()) {
-            if (typeIter.value().toString() == "m.space"_ls) {
-                return NeoChatRoomType::Space;
-            }
-        }
-        return NeoChatRoomType::Normal;
+        return NeoChatRoomType::typeForRoom(room);
     }
-    if (role == NotificationCountRole) {
-        return room->notificationCount();
+    if (role == ContextNotificationCountRole) {
+        return room->contextAwareNotificationCount();
     }
-    if (role == HighlightCountRole) {
-        return room->highlightCount();
+    if (role == HasHighlightNotificationsRole) {
+        return room->highlightCount() > 0 && room->contextAwareNotificationCount() > 0;
     }
     if (role == LastActiveTimeRole) {
         return room->lastActiveTime();
@@ -341,16 +241,11 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
     if (role == CurrentRoomRole) {
         return QVariant::fromValue(room);
     }
-    if (role == CategoryVisibleRole) {
-        return m_categoryVisibility.value(data(index, CategoryRole).toInt(), true);
-    }
     if (role == SubtitleTextRole) {
         if (room->lastEvent() == nullptr || room->lastEventIsSpoiler()) {
             return QString();
         }
-        EventHandler eventHandler;
-        eventHandler.setRoom(room);
-        eventHandler.setEvent(room->lastEvent());
+        EventHandler eventHandler(room, room->lastEvent());
         return eventHandler.subtitleText();
     }
     if (role == AvatarImageRole) {
@@ -363,11 +258,14 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
         return room->isSpace();
     }
     if (role == IsChildSpaceRole) {
-        return SpaceHierarchyCache::instance().isChildSpace(room->id());
+        return SpaceHierarchyCache::instance().isChild(room->id());
     }
     if (role == ReplacementIdRole) {
         return room->successorId();
     }
+    if (role == IsDirectChat) {
+        return room->isDirectChat();
+    }
 
     return QVariant();
 }
@@ -387,84 +285,24 @@ QHash<int, QByteArray> RoomListModel::roleNames() const
 {
     QHash<int, QByteArray> roles;
     roles[DisplayNameRole] = "displayName";
+    roles[EscapedDisplayNameRole] = "escapedDisplayName";
     roles[AvatarRole] = "avatar";
     roles[CanonicalAliasRole] = "canonicalAlias";
     roles[TopicRole] = "topic";
     roles[CategoryRole] = "category";
-    roles[NotificationCountRole] = "notificationCount";
-    roles[HighlightCountRole] = "highlightCount";
+    roles[ContextNotificationCountRole] = "contextNotificationCount";
+    roles[HasHighlightNotificationsRole] = "hasHighlightNotifications";
     roles[LastActiveTimeRole] = "lastActiveTime";
     roles[JoinStateRole] = "joinState";
     roles[CurrentRoomRole] = "currentRoom";
-    roles[CategoryVisibleRole] = "categoryVisible";
     roles[SubtitleTextRole] = "subtitleText";
     roles[IsSpaceRole] = "isSpace";
     roles[RoomIdRole] = "roomId";
     roles[IsChildSpaceRole] = "isChildSpace";
+    roles[IsDirectChat] = "isDirectChat";
     return roles;
 }
 
-QString RoomListModel::categoryName(int category)
-{
-    switch (category) {
-    case NeoChatRoomType::Invited:
-        return i18n("Invited");
-    case NeoChatRoomType::Favorite:
-        return i18n("Favorite");
-    case NeoChatRoomType::Direct:
-        return i18n("Direct Messages");
-    case NeoChatRoomType::Normal:
-        return i18n("Normal");
-    case NeoChatRoomType::Deprioritized:
-        return i18n("Low priority");
-    case NeoChatRoomType::Space:
-        return i18n("Spaces");
-    default:
-        return {};
-    }
-}
-
-QString RoomListModel::categoryIconName(int category)
-{
-    switch (category) {
-    case NeoChatRoomType::Invited:
-        return QStringLiteral("user-invisible");
-    case NeoChatRoomType::Favorite:
-        return QStringLiteral("favorite");
-    case NeoChatRoomType::Direct:
-        return QStringLiteral("dialog-messages");
-    case NeoChatRoomType::Normal:
-        return QStringLiteral("group");
-    case NeoChatRoomType::Deprioritized:
-        return QStringLiteral("object-order-lower");
-    case NeoChatRoomType::Space:
-        return QStringLiteral("group");
-    default:
-        return QStringLiteral("tools-report-bug");
-    }
-}
-
-void RoomListModel::setCategoryVisible(int category, bool visible)
-{
-    beginResetModel();
-    auto collapsedSections = NeoChatConfig::collapsedSections();
-    if (visible) {
-        collapsedSections.removeAll(category);
-    } else {
-        collapsedSections.push_back(category);
-    }
-    NeoChatConfig::setCollapsedSections(collapsedSections);
-    NeoChatConfig::self()->save();
-
-    m_categoryVisibility[category] = visible;
-    endResetModel();
-}
-
-bool RoomListModel::categoryVisible(int category) const
-{
-    return m_categoryVisibility.value(category, true);
-}
-
 NeoChatRoom *RoomListModel::roomByAliasOrId(const QString &aliasOrId)
 {
     for (const auto &room : std::as_const(m_rooms)) {
