@@ -1,15 +1,18 @@
 // SPDX-FileCopyrightText: 2021 Carl Schwan <carlschwan@kde.org>
-// SPDX-FileCopyrightText: 2021 Alexey Rusakov <TODO>
+// SPDX-FileCopyrightText: 2021 Alexey Rusakov <Kitsune-Ral@users.sf.net>
 // SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 #include "roommanager.h"
 
 #include "chatbarcache.h"
-#include "enums/delegatetype.h"
+#include "controller.h"
+#include "eventhandler.h"
 #include "models/timelinemodel.h"
 #include "neochatconfig.h"
 #include "neochatconnection.h"
 #include "neochatroom.h"
+#include "spacehierarchycache.h"
+#include "urlhelper.h"
 
 #include <KLocalizedString>
 #include <QDesktopServices>
@@ -27,18 +30,32 @@
 
 RoomManager::RoomManager(QObject *parent)
     : QObject(parent)
-    , m_currentRoom(nullptr)
-    , m_lastCurrentRoom(nullptr)
     , m_config(KSharedConfig::openStateConfig())
+    , m_roomListModel(new RoomListModel(this))
+    , m_sortFilterRoomListModel(new SortFilterRoomListModel(m_roomListModel, this))
+    , m_sortFilterSpaceListModel(new SortFilterSpaceListModel(m_roomListModel, this))
+    , m_roomTreeModel(new RoomTreeModel(this))
+    , m_sortFilterRoomTreeModel(new SortFilterRoomTreeModel(m_roomTreeModel, this))
     , m_timelineModel(new TimelineModel(this))
     , m_messageFilterModel(new MessageFilterModel(this, m_timelineModel))
     , m_mediaMessageFilterModel(new MediaMessageFilterModel(this, m_messageFilterModel))
 {
     m_lastRoomConfig = m_config->group(QStringLiteral("LastOpenRoom"));
+    m_lastSpaceConfig = m_config->group(QStringLiteral("LastOpenSpace"));
+    m_directChatsConfig = m_config->group(QStringLiteral("DirectChatsActive"));
 
     connect(this, &RoomManager::currentRoomChanged, this, [this]() {
         m_timelineModel->setRoom(m_currentRoom);
     });
+
+    connect(&Controller::instance(), &Controller::activeConnectionChanged, this, [this](NeoChatConnection *connection) {
+        setConnection(connection);
+    });
+    connect(this, &RoomManager::connectionChanged, this, [this]() {
+        m_roomListModel->setConnection(m_connection);
+        m_roomTreeModel->setConnection(m_connection);
+    });
+    connect(m_sortFilterSpaceListModel, &SortFilterSpaceListModel::layoutChanged, m_sortFilterRoomTreeModel, &SortFilterRoomTreeModel::invalidate);
 }
 
 RoomManager::~RoomManager()
@@ -56,6 +73,31 @@ NeoChatRoom *RoomManager::currentRoom() const
     return m_currentRoom;
 }
 
+RoomListModel *RoomManager::roomListModel() const
+{
+    return m_roomListModel;
+}
+
+SortFilterRoomListModel *RoomManager::sortFilterRoomListModel() const
+{
+    return m_sortFilterRoomListModel;
+}
+
+SortFilterSpaceListModel *RoomManager::sortFilterSpaceListModel() const
+{
+    return m_sortFilterSpaceListModel;
+}
+
+RoomTreeModel *RoomManager::roomTreeModel() const
+{
+    return m_roomTreeModel;
+}
+
+SortFilterRoomTreeModel *RoomManager::sortFilterRoomTreeModel() const
+{
+    return m_sortFilterRoomTreeModel;
+}
+
 TimelineModel *RoomManager::timelineModel() const
 {
     return m_timelineModel;
@@ -84,6 +126,11 @@ void RoomManager::resolveResource(const QString &idOrUri, const QString &action)
         return;
     }
 
+    if (uri.type() == Uri::NonMatrix && action == "qr"_ls) {
+        Q_EMIT externalUrl(uri.toUrl());
+        return;
+    }
+
     if (uri.type() != Uri::NonMatrix) {
         if (!m_connection) {
             return;
@@ -96,13 +143,15 @@ void RoomManager::resolveResource(const QString &idOrUri, const QString &action)
 
     const auto result = visitResource(m_connection, uri);
     if (result == Quotient::CouldNotResolve) {
-        Q_EMIT warning(i18n("Room not found"), i18n("There's no room %1 in the room list. Check the spelling and the account.", idOrUri));
+        if ((uri.type() == Uri::RoomAlias || uri.type() == Uri::RoomId) && action != "no_join"_ls) {
+            Q_EMIT askJoinRoom(uri.primaryId());
+        }
     } else { // Invalid cases should have been eliminated earlier
         Q_ASSERT(result == Quotient::UriResolved);
 
         if (uri.type() == Uri::RoomAlias || uri.type() == Uri::RoomId) {
-            connectSingleShot(m_connection, &Connection::newRoom, this, [this, uri](Room *room) {
-                enterRoom(dynamic_cast<NeoChatRoom *>(room));
+            connectSingleShot(m_connection.get(), &NeoChatConnection::newRoom, this, [this, uri](Room *room) {
+                resolveResource(room->id());
             });
         }
     }
@@ -116,6 +165,14 @@ void RoomManager::maximizeMedia(int index)
     Q_EMIT showMaximizedMedia(index);
 }
 
+void RoomManager::maximizeCode(const QVariantMap &author, const QDateTime &time, const QString &codeText, const QString &language)
+{
+    if (codeText.isEmpty()) {
+        return;
+    }
+    Q_EMIT showMaximizedCode(author, time, codeText, language);
+}
+
 void RoomManager::requestFullScreenClose()
 {
     Q_EMIT closeFullScreen();
@@ -126,22 +183,27 @@ void RoomManager::viewEventSource(const QString &eventId)
     Q_EMIT showEventSource(eventId);
 }
 
-void RoomManager::viewEventMenu(const QString &eventId,
-                                const QVariantMap &author,
-                                DelegateType::Type delegateType,
-                                const QString &plainText,
-                                const QString &htmlText,
-                                const QString &selectedText,
-                                const QString &mimeType,
-                                const FileTransferInfo &progressInfo)
+void RoomManager::viewEventMenu(const QString &eventId, NeoChatRoom *room, const QString &selectedText)
 {
-    if (delegateType == DelegateType::Image || delegateType == DelegateType::Video || delegateType == DelegateType::Audio
-        || delegateType == DelegateType::File) {
-        Q_EMIT showFileMenu(eventId, author, delegateType, plainText, mimeType, progressInfo);
+    const auto &event = **room->findInTimeline(eventId);
+    const auto eventHandler = EventHandler(room, &event);
+
+    if (eventHandler.getMediaInfo().contains("mimeType"_ls)) {
+        Q_EMIT showFileMenu(eventId,
+                            eventHandler.getAuthor(),
+                            eventHandler.messageComponentType(),
+                            eventHandler.getPlainBody(),
+                            eventHandler.getMediaInfo()["mimeType"_ls].toString(),
+                            room->fileTransferInfo(eventId));
         return;
     }
 
-    Q_EMIT showMessageMenu(eventId, author, delegateType, plainText, htmlText, selectedText);
+    Q_EMIT showMessageMenu(eventId,
+                           eventHandler.getAuthor(),
+                           eventHandler.messageComponentType(),
+                           eventHandler.getPlainBody(),
+                           eventHandler.getRichBody(),
+                           selectedText);
 }
 
 bool RoomManager::hasOpenRoom() const
@@ -175,96 +237,14 @@ void RoomManager::loadInitialRoom()
 void RoomManager::openRoomForActiveConnection()
 {
     if (!m_connection) {
-        return;
+        m_currentRoom = nullptr;
     }
-    // Read from last open room
-    QString roomId = m_lastRoomConfig.readEntry(m_connection->userId(), QString());
-
-    // TODO remove legacy check at some point.
-    if (roomId.isEmpty()) {
-        roomId = NeoChatConfig::self()->openRoom();
-    }
-
-    if (!roomId.isEmpty()) {
-        // Here we can cast because the controller has been configured to
-        // return NeoChatRoom instead of simple Quotient::Room
-        const auto room = qobject_cast<NeoChatRoom *>(m_connection->room(roomId));
-
-        if (room) {
-            if (room->isSpace()) {
-                enterSpaceHome(room);
-            } else {
-                enterRoom(room);
-            }
-        }
-    }
-}
-
-void RoomManager::enterRoom(NeoChatRoom *room)
-{
-    if (m_currentRoom && !m_currentRoom->editCache()->editId().isEmpty()) {
-        m_currentRoom->editCache()->setEditId({});
-    }
-    if (m_currentRoom && m_chatDocumentHandler) {
-        // We're doing these things here because it is critical that they are switched at the same time
-        if (m_chatDocumentHandler->document()) {
-            m_currentRoom->mainCache()->setSavedText(m_chatDocumentHandler->document()->textDocument()->toPlainText());
-            m_chatDocumentHandler->setRoom(room);
-            m_chatDocumentHandler->document()->textDocument()->setPlainText(room->mainCache()->savedText());
-            room->mainCache()->setText(room->mainCache()->savedText());
-        } else {
-            m_chatDocumentHandler->setRoom(room);
-        }
-    }
-    m_lastCurrentRoom = std::exchange(m_currentRoom, room);
-    Q_EMIT currentRoomChanged();
-
-    if (!m_lastCurrentRoom) {
-        Q_EMIT pushRoom(room, QString());
+    if (m_lastRoomConfig.readEntry(m_connection->userId(), QString()).isEmpty()) {
+        setCurrentRoom({});
     } else {
-        Q_EMIT replaceRoom(m_currentRoom, QString());
+        resolveResource(m_lastRoomConfig.readEntry(m_connection->userId(), QString()));
     }
-
-    if (room && room->timelineSize() == 0) {
-        room->getPreviousContent(20);
-    }
-
-    // Save last open room
-    m_lastRoomConfig.writeEntry(m_connection->userId(), room->id());
-}
-
-void RoomManager::openWindow(NeoChatRoom *room)
-{
-    // forward the call to QML
-    Q_EMIT openRoomInNewWindow(room);
-}
-
-void RoomManager::enterSpaceHome(NeoChatRoom *spaceRoom)
-{
-    if (!spaceRoom->isSpace()) {
-        return;
-    }
-    // If replacing a normal room message timeline make sure any edit is cancelled.
-    if (m_currentRoom && !m_currentRoom->editCache()->editId().isEmpty()) {
-        m_currentRoom->editCache()->setEditId({});
-    }
-    // Save the chatbar text for the current room if any before switching
-    if (m_currentRoom && m_chatDocumentHandler) {
-        if (m_chatDocumentHandler->document()) {
-            m_currentRoom->mainCache()->setSavedText(m_chatDocumentHandler->document()->textDocument()->toPlainText());
-        }
-    }
-    m_lastCurrentRoom = std::exchange(m_currentRoom, spaceRoom);
-    Q_EMIT currentRoomChanged();
-
-    if (!m_lastCurrentRoom) {
-        Q_EMIT pushSpaceHome(spaceRoom);
-    } else {
-        Q_EMIT replaceSpaceHome(m_currentRoom);
-    }
-
-    // Save last open room
-    m_lastRoomConfig.writeEntry(m_connection->userId(), spaceRoom->id());
+    setCurrentSpace(m_lastSpaceConfig.readEntry(m_connection->userId(), QString()), false);
 }
 
 UriResolveResult RoomManager::visitUser(User *user, const QString &action)
@@ -285,23 +265,36 @@ UriResolveResult RoomManager::visitUser(User *user, const QString &action)
     return Quotient::UriResolved;
 }
 
-void RoomManager::visitRoom(Room *room, const QString &eventId)
+void RoomManager::visitRoom(Room *r, const QString &eventId)
 {
-    auto neoChatRoom = qobject_cast<NeoChatRoom *>(room);
-    Q_ASSERT(neoChatRoom != nullptr);
+    auto room = qobject_cast<NeoChatRoom *>(r);
 
-    if (m_currentRoom) {
-        if (m_currentRoom->id() == room->id()) {
-            Q_EMIT goToEvent(eventId);
+    if (m_currentRoom && !m_currentRoom->editCache()->editId().isEmpty()) {
+        m_currentRoom->editCache()->setEditId({});
+    }
+    if (m_currentRoom && !m_currentRoom->isSpace() && m_chatDocumentHandler) {
+        // We're doing these things here because it is critical that they are switched at the same time
+        if (m_chatDocumentHandler->document()) {
+            m_currentRoom->mainCache()->setSavedText(m_chatDocumentHandler->document()->textDocument()->toPlainText());
+            m_chatDocumentHandler->setRoom(room);
+            if (room) {
+                m_chatDocumentHandler->document()->textDocument()->setPlainText(room->mainCache()->savedText());
+                room->mainCache()->setText(room->mainCache()->savedText());
+            }
         } else {
-            m_lastCurrentRoom = std::exchange(m_currentRoom, neoChatRoom);
-            Q_EMIT currentRoomChanged();
-            Q_EMIT replaceRoom(neoChatRoom, eventId);
+            m_chatDocumentHandler->setRoom(room);
         }
+    }
+
+    if (!room) {
+        setCurrentRoom({});
+        return;
+    }
+
+    if (m_currentRoom && m_currentRoom->id() == room->id()) {
+        Q_EMIT goToEvent(eventId);
     } else {
-        m_lastCurrentRoom = std::exchange(m_currentRoom, neoChatRoom);
-        Q_EMIT currentRoomChanged();
-        Q_EMIT pushRoom(neoChatRoom, eventId);
+        setCurrentRoom(room->id());
     }
 }
 
@@ -310,8 +303,8 @@ void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAli
     auto job = account->joinRoom(roomAliasOrId, viaServers);
     connectSingleShot(job, &Quotient::BaseJob::finished, this, [this, account](Quotient::BaseJob *finish) {
         if (finish->status() == Quotient::BaseJob::Success) {
-            connectSingleShot(account, &Quotient::Connection::newRoom, this, [this](Quotient::Room *room) {
-                enterRoom(dynamic_cast<NeoChatRoom *>(room));
+            connectSingleShot(account, &NeoChatConnection::newRoom, this, [this](Quotient::Room *room) {
+                resolveResource(room->id());
             });
         } else {
             Q_EMIT warning(i18n("Failed to join room"), finish->errorString());
@@ -319,7 +312,7 @@ void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAli
     });
 }
 
-void RoomManager::knockRoom(Quotient::Connection *account, const QString &roomAliasOrId, const QString &reason, const QStringList &viaServers)
+void RoomManager::knockRoom(NeoChatConnection *account, const QString &roomAliasOrId, const QString &reason, const QStringList &viaServers)
 {
     auto *const job = account->callApi<KnockRoomJob>(roomAliasOrId, viaServers, reason);
     // Upon completion, ensure a room object is created in case it hasn't come
@@ -328,7 +321,7 @@ void RoomManager::knockRoom(Quotient::Connection *account, const QString &roomAl
     // to overtake clients that may add their own slots to finished().
     connectSingleShot(job, &BaseJob::finished, this, [this, job, account] {
         if (job->status() == Quotient::BaseJob::Success) {
-            connectSingleShot(account, &Quotient::Connection::newRoom, this, [this](Quotient::Room *room) {
+            connectSingleShot(account, &NeoChatConnection::newRoom, this, [this](Quotient::Room *room) {
                 Q_EMIT currentRoom()->showMessage(NeoChatRoom::Info, i18n("You requested to join '%1'", room->name()));
             });
         } else {
@@ -339,42 +332,22 @@ void RoomManager::knockRoom(Quotient::Connection *account, const QString &roomAl
 
 bool RoomManager::visitNonMatrix(const QUrl &url)
 {
-#ifdef Q_OS_ANDROID
-    if (!QDesktopServices::openUrl(url)) {
-        Q_EMIT warning(i18n("No application for the link"), i18n("Your operating system could not find an application for the link."));
-    }
-#else
-    auto *job = new KIO::OpenUrlJob(url);
-    connect(job, &KJob::finished, this, [this](KJob *job) {
-        if (job->error()) {
-            Q_EMIT warning(i18n("Could not open URL"), job->errorString());
-        }
-    });
-    job->start();
-#endif
+    UrlHelper().openUrl(url);
     return true;
 }
 
-void RoomManager::reset()
-{
-    m_arg = QString();
-    m_currentRoom = nullptr;
-    m_lastCurrentRoom = nullptr;
-    Q_EMIT currentRoomChanged();
-}
-
 void RoomManager::leaveRoom(NeoChatRoom *room)
 {
     if (!room) {
         return;
     }
-    if (m_lastCurrentRoom && room->id() == m_lastCurrentRoom->id()) {
-        m_lastCurrentRoom = nullptr;
-    }
+
     if (m_currentRoom && m_currentRoom->id() == room->id()) {
-        m_currentRoom = m_lastCurrentRoom;
-        m_lastCurrentRoom = nullptr;
-        Q_EMIT currentRoomChanged();
+        setCurrentRoom({});
+    }
+
+    if (m_currentSpaceId == room->id()) {
+        setCurrentSpace({});
     }
 
     room->forget();
@@ -392,11 +365,6 @@ void RoomManager::setChatDocumentHandler(ChatDocumentHandler *handler)
     Q_EMIT chatDocumentHandlerChanged();
 }
 
-NeoChatConnection *RoomManager::connection() const
-{
-    return m_connection;
-}
-
 void RoomManager::setConnection(NeoChatConnection *connection)
 {
     if (m_connection == connection) {
@@ -406,4 +374,87 @@ void RoomManager::setConnection(NeoChatConnection *connection)
     Q_EMIT connectionChanged();
 }
 
+void RoomManager::setCurrentSpace(const QString &spaceId, bool setRoom)
+{
+    m_currentSpaceId = spaceId;
+
+    // This need to happen before the signal so TreeView.expandRecursively() can work nicely.
+    m_sortFilterRoomTreeModel->setActiveSpaceId(m_currentSpaceId);
+    m_sortFilterRoomTreeModel->setMode(m_currentSpaceId == QLatin1String("DM") ? SortFilterRoomTreeModel::DirectChats : SortFilterRoomTreeModel::Rooms);
+
+    Q_EMIT currentSpaceChanged();
+    m_lastSpaceConfig.writeEntry(m_connection->userId(), spaceId);
+
+    if (!setRoom) {
+        return;
+    }
+    if (spaceId.length() > 3) {
+        resolveResource(spaceId, "no_join"_ls);
+    } else {
+        visitRoom({}, {});
+    }
+}
+
+void RoomManager::setCurrentRoom(const QString &roomId)
+{
+    if (roomId.isEmpty()) {
+        m_currentRoom = nullptr;
+    } else {
+        m_currentRoom = dynamic_cast<NeoChatRoom *>(m_connection->room(roomId));
+    }
+    Q_EMIT currentRoomChanged();
+    if (m_connection) {
+        m_lastRoomConfig.writeEntry(m_connection->userId(), roomId);
+    }
+    if (roomId.isEmpty()) {
+        return;
+    }
+    if (m_currentRoom->isSpace()) {
+        return;
+    }
+    if (m_currentRoom->isDirectChat()) {
+        if (m_currentSpaceId != "DM"_ls) {
+            setCurrentSpace("DM"_ls, false);
+        }
+        return;
+    }
+    const auto &parentSpaces = SpaceHierarchyCache::instance().parentSpaces(roomId);
+    if (parentSpaces.contains(m_currentSpaceId)) {
+        return;
+    }
+    static auto config = NeoChatConfig::self();
+    if (config->allRoomsInHome()) {
+        setCurrentSpace({}, false);
+        return;
+    }
+    if (const auto &parent = m_connection->room(m_currentRoom->canonicalParent())) {
+        for (const auto &parentParent : SpaceHierarchyCache::instance().parentSpaces(parent->id())) {
+            if (SpaceHierarchyCache::instance().parentSpaces(parentParent).isEmpty()) {
+                setCurrentSpace(parentParent, false);
+                return;
+            }
+        }
+        setCurrentSpace(parent->id(), false);
+        return;
+    }
+    for (const auto &space : parentSpaces) {
+        if (SpaceHierarchyCache::instance().parentSpaces(space).isEmpty()) {
+            setCurrentSpace(space, false);
+            return;
+        }
+    }
+    setCurrentSpace({}, false);
+}
+
+QString RoomManager::currentSpace() const
+{
+    return m_currentSpaceId;
+}
+
+void RoomManager::resetState()
+{
+    setCurrentRoom({});
+    setCurrentSpace({});
+}
+
 #include "moc_roommanager.cpp"
