@@ -1,11 +1,43 @@
 // SPDX-FileCopyrightText: 2023 James Graham <james.h.graham@protonmail.com>
 // SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
+#pragma once
+
 #include <QColor>
 #include <QGuiApplication>
 #include <QPalette>
+#include <QQmlEngine>
 #include <QRegularExpression>
 
+#include <Quotient/user.h>
+
+class QmlUtils : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
+
+public:
+    static QmlUtils *create(QQmlEngine *, QJSEngine *)
+    {
+        QQmlEngine::setObjectOwnership(&instance(), QQmlEngine::CppOwnership);
+        return &instance();
+    }
+
+    static QmlUtils &instance()
+    {
+        static QmlUtils _instance;
+        return _instance;
+    }
+
+    Q_INVOKABLE QVariantMap getUser(Quotient::User *user) const;
+    Q_INVOKABLE bool isValidJson(const QByteArray &json);
+
+private:
+    QmlUtils() = default;
+};
+
+
 namespace Utils
 {
 
